/*
 * $Id: 8d28fd679f79e4e18977186e7ff58acfaf9633af $
 *
 * This file is part of ntldap.
 *
 * Copyright (c) 2006, 2007, 2008 Niklas Olmes <niklas@noxa.de>
 *                                http://www.nevm.de/ntldap/
 *                                NEVM-RIPE
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include <unistd.h>
#include <syslog.h>

#include "ldap.h"

#include "ntl.h"
#include "ntlc.h"
#include "drone.h"

extern const char *ntl_error_str;

static const char rcsid[] = "$Id: 8d28fd679f79e4e18977186e7ff58acfaf9633af $";

void
drone_generic(const NTL_CFG *cfg,
    void (*startfn)(void),
    void (*checkfn)(const NTL_ATTR *),
    void (*endfn)(void),
    const char *fltr, const char **attrs,
    const char *ident, int interval)
{
	NTL_ATTR *attr;
	LDAP *ld;
	LDAPMessage *res, *entry;
	unsigned long runs, actruns;
	int i;

	openlog(ident, LOG_PID | LOG_PERROR, LOG_USER);

	runs = actruns = 0;

	for (;;) {
		ld = NULL;
		res = entry = NULL;
		runs++;

		if ((ld = ntl_connect(cfg)) == NULL)
			syslog(LOG_ERR, "Connection failed: %s", ntl_error_str);
		else if ((res = ntl_search(ld, cfg->basedn, LDAP_SCOPE_SUBTREE, fltr, attrs)) == NULL)
			syslog(LOG_ERR, "Search error: %s", ntl_error_str);
		else if ((entry = ldap_first_entry(ld, res)) == NULL)
			syslog(LOG_WARNING, "Nothing found");
		else {
			startfn();

			for (
				i = 1;
				entry != NULL;
				entry = ldap_next_entry(ld, entry), i++
			) {
				attr = ntl_entry(ld, entry);
				checkfn(attr);
				ntl_entry_free(attr);
			}

			actruns++;
			endfn();
		}

		if (runs == 1 || runs % 100 == 0)
			syslog(LOG_INFO, "Survived %lu runs (%lu actual) and counting.", runs, actruns);

		if (res != NULL)
			(void)ldap_msgfree(res);
		if (ld != NULL)
			(void)ldap_unbind(ld);

		(void)sleep((unsigned int)interval);
	}
	/* NOTREACHED */
}
