/*
 * $Id: 666c563c09f4341330756ceff9d1f50debaf0812 $
 *
 * This file is part of ntldap.
 *
 * Copyright (c) 2006, 2007, 2008 Niklas Olmes <niklas@noxa.de>
 *                                http://www.nevm.de/ntldap/
 *                                NEVM-RIPE
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <err.h>

#include "nt_alloc.h"

static const char rcsid[] = "$Id: 666c563c09f4341330756ceff9d1f50debaf0812 $";

void *
nt_calloc(size_t n, size_t size)
{
	void *p;

	if ((p = calloc(n, size)) == NULL)
		err(70, "calloc(%lu, %lu) failed", (unsigned long)n, (unsigned long)size);
	return p;
}

void *
nt_malloc(size_t size)
{
	void *p;

	if ((p = malloc(size)) == NULL)
		err(71, "malloc(%lu) failed", (unsigned long)size);
	(void)memset(p, 0, size);
	return p;
}

void *
nt_realloc(void *p, size_t size)
{
	char *p2;

	if ((p2 = realloc(p, size)) == NULL)
		err(72, "realloc(.., %lu) failed", (unsigned long)size);
	return p2;
}

char *
nt_strdup(const char *str)
{
	char *p;

	if ((p = strdup(str)) == NULL)
		err(73, "strdup() (strlen == %lu) failed", (unsigned long)strlen(str));
	return p;
}

void
nt_asprintf(char **ret, const char *fmt, ...)
{
	va_list ap;
	char buf[65536];

	va_start(ap, fmt);
	if (vsnprintf(buf, sizeof(buf), fmt, ap) < 0)
		err(74, "vsnprintf(buf, %lu, fmt, ...) failed", (unsigned long)sizeof(buf));
	va_end(ap);

	*ret = nt_strdup(buf);
}
