/*
 * $Id: 839852886c9cb30981c83fb8ed26c123ddd52d85 $
 *
 * This file is part of ntldap.
 *
 * Copyright (c) 2006, 2007, 2008 Niklas Olmes <niklas@noxa.de>
 *                                http://www.nevm.de/ntldap/
 *                                NEVM-RIPE
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#ifndef HAD_LIBNTLDAP_NTG_H
#define HAD_LIBNTLDAP_NTG_H

#include "ldap.h"

#include "ncurses.h"
#include "menu.h"
#include "form.h"
#include "cdk/cdk.h"

#include "ntl.h"
#include "ntls.h"

/* minimum dimensions of terminal for ntg* to be useful */
#define NTG_MIN_HEIGHT	24
#define NTG_MIN_WIDTH	72

#define NTG_KEYHELP_LINES	1

#define NTG_E_INIT	1
#define NTG_E_INPUT	10
#define NTG_E_WIN	20
#define NTG_E_INTERN	127

#define NTG_PAGER_RET_NEXT	1
#define NTG_PAGER_RET_PREV	-1
#define NTG_PAGER_RET_EXIT	0
#define NTG_PAGER_RET_WRITE	2
#define NTG_PAGER_RET_DEL	3
#define NTG_PAGER_RET_CREATE_FROM_TPL	4
#define NTG_PAGER_RET_CREATE_FROM_OC	5
#define NTG_PAGER_RET_LDIF	6
#define NTG_PAGER_RET_RELOAD	7
#define NTG_PAGER_RET_SEARCH	8
#define NTG_PAGER_RET_PASSWD	9
#define NTG_PAGER_RET_SUICIDE	10
#define NTG_PAGER_RET_MODRDN	11
#define NTG_PAGER_RET_COPY	12
#define NTG_PAGER_RET_MOVE	13
#define NTG_PAGER_RET_CREATE_ATTR	14

#define NTG_MENU_LDAP		1
#define NTG_MENU_SIMPLE		2

#define NTG_MENU_RET_REFRESH	-1
#define NTG_MENU_RET_REFSEARCH	-2
#define NTG_MENU_RET_SUBSEARCH	-3
#define NTG_MENU_RET_CHBASE	-4
#define NTG_MENU_RET_SCOPE	-5
#define NTG_MENU_RET_DIVE	-6
#define NTG_MENU_RET_CREATE_FROM_TPL	-7
#define NTG_MENU_RET_CREATE_FROM_OC	-8
#define NTG_MENU_RET_DEL	-9
#define NTG_MENU_RET_DELM	-10
#define NTG_MENU_RET_COPY	-11
#define NTG_MENU_RET_COPYM	-12
#define NTG_MENU_RET_MOVE	-13
#define NTG_MENU_RET_MOVEM	-14
#define NTG_MENU_RET_SUICIDE	-15
#define NTG_MENU_RET_LDIF	-16
#define NTG_MENU_RET_CHSORT	-17
#define NTG_MENU_RET_RETURN	-18

#define C_DEFAULT_BG	(COLOR_BLACK)
#define C_NORMAL_NUM	1UL
#define C_NORMAL	COLOR_PAIR(C_NORMAL_NUM)
#define C_TEXT_NUM	2UL
#define C_TEXT		COLOR_PAIR(C_TEXT_NUM)
#define C_STRONG_NUM	3UL
#define C_STRONG	COLOR_PAIR(C_STRONG_NUM)
#define C_EM_NUM	4UL
#define C_EM		COLOR_PAIR(C_EM_NUM)
#define C_SUCCESS_NUM	5UL
#define C_SUCCESS	COLOR_PAIR(C_SUCCESS_NUM)
#define C_ERROR_NUM	6UL
#define C_ERROR		COLOR_PAIR(C_ERROR_NUM)
#define C_INPUT_NUM	7UL
#define C_INPUT		COLOR_PAIR(C_INPUT_NUM)
#define C_INFO_NUM	8UL
#define C_INFO		COLOR_PAIR(C_INFO_NUM)
#define C_MSG_NUM	9UL
#define C_MSG		COLOR_PAIR(C_MSG_NUM)
#define C_BLUE_NUM	10UL
#define C_BLUE		COLOR_PAIR(C_BLUE_NUM)

typedef struct ntg_keys {
	int cr_tpl;
	int cr_oc;
	int cr_attr;
	int refresh;
	int del;
	int del_many;
	int display_auto;
	int display_tpl;
	int dive;
	int ret;
	int search;
	int search_dn;
	int refsearch;
	int subsearch;
	int chg_base;
	int chg_sort;
	int tgl_operational;
	int scope;
	int suicide;
	int write;
	int ldif;
	int edit;
	int tip;
	int password;
	int modrdn;
	int copy;
	int copy_many;
	int move;
	int move_many;
	int validate;
	int open_editor;
	int view_attr;
	char **keyhelp_menu;
	char **keyhelp_form;
	char **keyhelp_search;
} NTG_KEYS;

typedef struct ntg_cdkmenu {
	CDKLABEL *label;
	int numentries;
	char *entries[MAX_MENU_ITEMS][MAX_SUB_ITEMS];
	char *info[MAX_MENU_ITEMS][MAX_SUB_ITEMS];
	int numsubs[MAX_MENU_ITEMS];
	int loc[MAX_MENU_ITEMS];
	int mapping[MAX_MENU_ITEMS][MAX_SUB_ITEMS];
} NTG_CDKMENU;

typedef struct ntg_ctx {
	WINDOW *rootwin;
	WINDOW *topwin;
	WINDOW *mainwin;
	WINDOW *bottomwin;
	CDKSCREEN *cdkscreen;
	CDKSCREEN *cdkrootscreen;
	CDKLABEL *cdklabel;
	CDKLABEL *cdktip;
	NTG_CDKMENU **cdkmenus;
	LDAP *ld;
	NTL_CFG *cfg;
	NTLS_SCHEMA *schema;
	NTG_KEYS *keys;
	void *tmp;
	int tmpflags;
	int tmpreturn;
	int oldcursor;
	int sortleft;
	int mainwin_maxx;
	int mainwin_maxy;
	int stdscr_maxx;
	int stdscr_maxy;
	const char **keyhelp;
	char title[256];
	char subtitle[256];
} NTG_CTX;

NTG_CTX *ntg_ctx_init(void);
void ntg_ctx_destroy(NTG_CTX *);
void ntg_cleanup(void);

int ntg_init_curses(NTG_CTX *);
int ntg_init_ldap(NTG_CTX *);
int ntg_init_schema(NTG_CTX *);

void ntg_fail(NTG_CTX *, const char *, int);
void ntg_refresh(NTG_CTX *);
void ntg_subtitle(NTG_CTX *, const char *);
void ntg_title(NTG_CTX *, const char *);

int ntg_select(NTG_CTX *, const char *, const char **);
int *ntg_select_multi(NTG_CTX *, const char *, const char **);
int ntg_newobject_tpl(NTG_CTX *, const char *);
int ntg_newobject_oc(NTG_CTX *, const char *);

MENU *ntg_menu(NTG_CTX *, ITEM **);
ITEM *ntg_menu_driver(NTG_CTX *, MENU *, int);

ITEM **ntg_items(const char **, const char **);
void ntg_items_free(ITEM **);
void ntg_set_edit(FORM *, int);
void ntg_set_virgin(FORM *);
int ntg_form_driver(NTG_CTX *, FORM *);
int ntg_form_driver_super(FORM *, int);

int ntg_open_shell(NTG_CTX *, const char *);
char *ntg_open_editor(NTG_CTX *, const char *);

char **ntg_strnl_to_strstr(const char *);
void ntg_strstr_free(char **);
void ntg_va(NTG_CTX *, void fn(NTG_CTX *, const char **), const char *, va_list);

char *ntg_input(NTG_CTX *, const char *, const char *);
void ntg_popup(NTG_CTX *, const char **, const char *);
void ntg_error_ldap(NTG_CTX *, const char *, const char *, int);
void ntg_error(NTG_CTX *, const char **);
void ntg_error_va(NTG_CTX *, const char *, ...);
void ntg_success(NTG_CTX *, const char **);
void ntg_success_va(NTG_CTX *, const char *, ...);
void ntg_message(NTG_CTX *, const char **);
void ntg_message_va(NTG_CTX *, const char *, ...);
void ntg_display(NTG_CTX *, const char **);
void ntg_display_va(NTG_CTX *, const char *, ...);
void ntg_tip_up(NTG_CTX *, const char **);
void ntg_tip_down(NTG_CTX *);
void ntg_attribute_tip(NTG_CTX *, const char *);
void ntg_info_up(NTG_CTX *, const char **);
void ntg_info_down(NTG_CTX *);
int ntg_yesno(NTG_CTX *, const char **);

int ntg_search_select(NTG_CTX *, LDAP *, LDAPMessage *, const char *, const char *, int (*)(NTG_CTX *, const char *, const char *, int));
int ntg_search(NTG_CTX *, const char *, int, const char *, int (*)(NTG_CTX *, const char *, const char *, int));

int ntg_display_ldif(NTG_CTX *, const char *);

const NTL_TPLC *ntg_get_tplc(NTG_CTX *, const char **);

int ntg_pager(NTG_CTX *, FIELD **);
int ntg_pager_generic(NTG_CTX *, const char *, const char *, int);
int ntg_pager_write(NTG_CTX *, const char *, FIELD **);

int ntg_create_obj_from_tpl(NTG_CTX *, const char *, const NTL_TPLC *);
int ntg_create_attr(NTG_CTX *, const char *, const NTL_TPLC *);
int ntg_passwd(NTG_CTX *, const char *);
int ntg_modrdn(NTG_CTX *, const char *);
int ntg_del(NTG_CTX *, const char *);
int ntg_copy_base(NTG_CTX *, const char *, const char *);
int ntg_copy(NTG_CTX *, const char *, const char *);
int ntg_move_base(NTG_CTX *, const char *, const char *);
int ntg_move(NTG_CTX *, const char *, const char *);

void ntg_keyhelp(NTG_CTX *, const char **);

FIELD **ntg_genfields(NTG_CTX *ctx, const NTL_ATTR *, const char *, const NTL_TPL **, int, int, int);
FIELD **ntg_genfields_auto(NTG_CTX *ctx, const NTL_ATTR *, const char *, int, int, int);
void ntg_fields_free(FIELD **);

#endif
