/*
 * $Id: f850668e8011a7ce6a8f1e3502650ce2741c9a0d $
 *
 * This file is part of ntldap.
 *
 * Copyright (c) 2006, 2007, 2008 Niklas Olmes <niklas@noxa.de>
 *                                http://www.nevm.de/ntldap/
 *                                NEVM-RIPE
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ncurses.h"

#include "ntg_keys.h"
#include "ntg.h"
#include "ntlc.h"
#include "nt_alloc.h"

static const char rcsid[] = "$Id: f850668e8011a7ce6a8f1e3502650ce2741c9a0d $";

int ntg_keys_error = 0;
int ntg_keys_error_line = 0;
char *ntg_keys_error_file = NULL;
const char *ntg_keys_error_str = NULL;

#define KEY_MOD		0x80

NTG_KEYS *
ntg_keys_init_default(void)
{
	NTG_KEYS *keys;

	keys = nt_calloc(1, sizeof(NTG_KEYS));

	keys->cr_tpl = KEY_MOD + 'T';		/* Mod-T */
	keys->cr_oc = KEY_MOD + 'O';		/* Mod-O */
	keys->cr_attr = KEY_MOD + 'A';		/* Mod-A */
	keys->refresh = 0x12;			/* ^R */
	keys->del = KEY_MOD + 'd';		/* Mod-d */
	keys->del_many = KEY_MOD + 'D';		/* Mod-D */
	keys->display_auto = '\n';		/* Return */
	keys->display_tpl = KEY_BACKSPACE;	/* Backspace */
	keys->dive = 0x04;			/* ^D */
	keys->ret = 0x18;			/* ^X */
	keys->search = '/';			/* / */
	keys->search_dn = '?';			/* ? */
	keys->refsearch = ';';			/* ; */
	keys->subsearch = 0x1f;			/* ^/ */
	keys->chg_base = KEY_MOD + 'b';		/* Mod-b */
	keys->chg_sort = '=';			/* = */
	keys->tgl_operational = '+';		/* + */
	keys->scope = '@';			/* @ */
	keys->suicide = KEY_F(10);		/* F10 */
	keys->write = KEY_MOD + 'w';		/* Mod-w */
	keys->ldif = KEY_MOD + 'L';		/* Mod-L */
	keys->edit = KEY_MOD + 'e';		/* Mod-e */
	keys->tip = 0x14;			/* ^T */
	keys->password = KEY_MOD + 'P';		/* Mod-P */
	keys->modrdn = KEY_MOD + 'R';		/* Mod-R */
	keys->copy = KEY_MOD + 'c';		/* Mod-c */
	keys->copy_many = KEY_MOD + 'C';	/* Mod-C */
	keys->move = KEY_MOD + 'm';		/* Mod-m */
	keys->move_many = KEY_MOD + 'M';	/* Mod-M */
	keys->validate = KEY_MOD + 'V';		/* Mod-V */
	keys->open_editor = KEY_MOD + 'E';	/* Mod-E */
	keys->view_attr = KEY_MOD + 'v';	/* Mod-v */

	keys->keyhelp_menu = nt_calloc(NTG_KEYHELP_LINES + 1, sizeof(char *));
	keys->keyhelp_menu[0] = nt_strdup("^X=Ret ESC=Esc F10=Quit M-ESC=Menu | Ret=Go");
	keys->keyhelp_menu[1] = NULL;

	keys->keyhelp_search = nt_calloc(NTG_KEYHELP_LINES + 1, sizeof(char *));
	keys->keyhelp_search[0] = nt_strdup("^X=Ret ESC=Esc F10=Quit M-ESC=Menu | Ret/Bsp=Show ^D=Dive ^R=Refr");
	keys->keyhelp_search[1] = NULL;

	keys->keyhelp_form = nt_calloc(NTG_KEYHELP_LINES + 1, sizeof(char *));
	keys->keyhelp_form[0] = nt_strdup("^X=Ret ESC=Esc F10=Quit | M-e=Edit M-w=Write M-A/T/O=New");
	keys->keyhelp_form[1] = NULL;

	return keys;
}

int
ntg_keys_cfg_read(NTG_KEYS *keys, const char *fn)
{
	FILE *fp;
	char rfn[256];
	char b[512];
	char obuf[64];
	char *p;
	int val;
	int line;

	ntg_keys_error = 0;
	ntg_keys_error_line = 0;
	ntg_keys_error_file = NULL;
	ntg_keys_error_str = NULL;

	if (fn == NULL)
		(void)snprintf(rfn, sizeof(rfn), "%s/%s", getenv("HOME"), NT_CFG_DEFAULT);
	else
		(void)snprintf(rfn, sizeof(rfn), "%s", fn);

	if ((fp = fopen(rfn, "r")) == NULL) {
		free(rfn);
		ntg_keys_error = 1;
		ntg_keys_error_str = "unable to open file";
		ntg_keys_error_file = nt_strdup(rfn);
		return 1;
	}

	/* skip until "keys" section */
	line = 1;
	while ((p = fgets(b, sizeof(b), fp)) != NULL && strncmp(b, "[keys]", strlen("[keys]")) != 0)
		line++;

	if (p == NULL) {
		(void)fclose(fp);
		return 0;
	}

	while (fgets(b, sizeof(b), fp) != NULL && *b != '[') {
		line++;
		(void)sscanf(b, "%63s %d", obuf, &val);
		if (*b == '\n')
			continue;

		if (strcmp(obuf, "cr_tpl") == 0)
			keys->cr_tpl = val;
		else if (strcmp(obuf, "cr_oc") == 0)
			keys->cr_oc = val;
		else if (strcmp(obuf, "cr_attr") == 0)
			keys->cr_attr = val;
		else if (strcmp(obuf, "refresh") == 0)
			keys->refresh = val;
		else if (strcmp(obuf, "del") == 0)
			keys->del = val;
		else if (strcmp(obuf, "del_many") == 0)
			keys->del_many = val;
		else if (strcmp(obuf, "display_auto") == 0)
			keys->display_auto = val;
		else if (strcmp(obuf, "display_tpl") == 0)
			keys->display_tpl = val;
		else if (strcmp(obuf, "dive") == 0)
			keys->dive = val;
		else if (strcmp(obuf, "ret") == 0)
			keys->ret = val;
		else if (strcmp(obuf, "search") == 0)
			keys->search = val;
		else if (strcmp(obuf, "search_dn") == 0)
			keys->search_dn = val;
		else if (strcmp(obuf, "refsearch") == 0)
			keys->refsearch = val;
		else if (strcmp(obuf, "subsearch") == 0)
			keys->subsearch = val;
		else if (strcmp(obuf, "chg_base") == 0)
			keys->chg_base = val;
		else if (strcmp(obuf, "chg_sort") == 0)
			keys->chg_sort = val;
		else if (strcmp(obuf, "tgl_operational") == 0)
			keys->tgl_operational = val;
		else if (strcmp(obuf, "scope") == 0)
			keys->scope = val;
		else if (strcmp(obuf, "suicide") == 0)
			keys->suicide = val;
		else if (strcmp(obuf, "write") == 0)
			keys->write = val;
		else if (strcmp(obuf, "ldif") == 0)
			keys->ldif = val;
		else if (strcmp(obuf, "edit") == 0)
			keys->edit = val;
		else if (strcmp(obuf, "tip") == 0)
			keys->tip = val;
		else if (strcmp(obuf, "password") == 0)
			keys->password = val;
		else if (strcmp(obuf, "modrdn") == 0)
			keys->modrdn = val;
		else if (strcmp(obuf, "copy") == 0)
			keys->copy = val;
		else if (strcmp(obuf, "copy_many") == 0)
			keys->copy_many = val;
		else if (strcmp(obuf, "move") == 0)
			keys->move = val;
		else if (strcmp(obuf, "move_many") == 0)
			keys->move_many = val;
		else if (strcmp(obuf, "validate") == 0)
			keys->validate = val;
		else if (strcmp(obuf, "open_editor") == 0)
			keys->open_editor = val;
		else if (strcmp(obuf, "view_attr") == 0)
			keys->view_attr = val;
		else if (strcmp(obuf, "keyhelp_form") == 0) {
			if (val < 1 || val > NTG_KEYHELP_LINES) {
				ntg_keys_error = 3;
				ntg_keys_error_str = "index for keyhelp_form out of range 1 <= x <= NTG_KEYHELP_LINES in section \"keys\" of config file";
				ntg_keys_error_line = line;
				ntg_keys_error_file = nt_strdup(rfn);
				(void)fclose(fp);
				return 3;
			}
			if (fgets(b, sizeof(b), fp) != NULL) {
				if (keys->keyhelp_form[val - 1] != NULL)
					free(keys->keyhelp_form[val - 1]);
				keys->keyhelp_form[val - 1] = nt_strdup(b);
			} else
				break;
			line++;
		} else if (strcmp(obuf, "keyhelp_search") == 0) {
			if (val < 1 || val > NTG_KEYHELP_LINES) {
				ntg_keys_error = 4;
				ntg_keys_error_str = "index for keyhelp_search out of range 1 <= x <= NTG_KEYHELP_LINES in section \"keys\" of config file";
				ntg_keys_error_line = line;
				ntg_keys_error_file = nt_strdup(rfn);
				(void)fclose(fp);
				return 4;
			}
			if (fgets(b, sizeof(b), fp) != NULL) {
				if (keys->keyhelp_search[val - 1] != NULL)
					free(keys->keyhelp_search[val - 1]);
				keys->keyhelp_search[val - 1] = nt_strdup(b);
			} else
				break;
			line++;
		} else if (strcmp(obuf, "keyhelp_menu") == 0) {
			if (val < 1 || val > NTG_KEYHELP_LINES) {
				ntg_keys_error = 5;
				ntg_keys_error_str = "index for keyhelp_menu out of range 1 <= x <= NTG_KEYHELP_LINES in section \"keys\" of config file";
				ntg_keys_error_line = line;
				ntg_keys_error_file = nt_strdup(rfn);
				(void)fclose(fp);
				return 5;
			}
			if (fgets(b, sizeof(b), fp) != NULL) {
				if (keys->keyhelp_menu[val - 1] != NULL)
					free(keys->keyhelp_menu[val - 1]);
				keys->keyhelp_menu[val - 1] = nt_strdup(b);
			} else
				break;
			line++;
		} else if (*obuf != '#') {
			ntg_keys_error = 2;
			ntg_keys_error_str = "unknown option in section \"keys\" of config file";
			ntg_keys_error_line = line;
			ntg_keys_error_file = nt_strdup(rfn);
			(void)fclose(fp);
			return 2;
		}
	}

	(void)fclose(fp);
	return 0;
}

void
ntg_keys_cfg_free(NTG_KEYS *keys)
{
	int i;

	if (keys == NULL)
		return;

	if (keys->keyhelp_menu != NULL) {
		for (i = 0; i < NTG_KEYHELP_LINES && keys->keyhelp_menu[i] != NULL; i++)
			free(keys->keyhelp_menu[i]);
		free(keys->keyhelp_menu);
	}
	if (keys->keyhelp_search != NULL) {
		for (i = 0; i < NTG_KEYHELP_LINES && keys->keyhelp_search[i] != NULL; i++)
			free(keys->keyhelp_search[i]);
		free(keys->keyhelp_search);
	}
	if (keys->keyhelp_form != NULL) {
		for (i = 0; i < NTG_KEYHELP_LINES && keys->keyhelp_form[i] != NULL; i++)
			free(keys->keyhelp_form[i]);
		free(keys->keyhelp_form);
	}
	free(keys);
}

void
ntg_keys_perror(const char *s)
{

	if (s != NULL)
		(void)fprintf(stderr, "%s: ", s);
	(void)fprintf(stderr, "error reading GUI keyboard config: (%d) %s (%s, line #%d)\n",
	    ntg_keys_error, ntg_keys_error_str, ntg_keys_error_file, ntg_keys_error_line);

	if (ntg_keys_error_file != NULL)
		free(ntg_keys_error_file);
}
