/*
 * $Id: 438024f3814d6719bb6c50806da03226a6b7be56 $
 *
 * This file is part of ntldap.
 *
 * Copyright (c) 2006, 2007, 2008 Niklas Olmes <niklas@noxa.de>
 *                                http://www.nevm.de/ntldap/
 *                                NEVM-RIPE
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <err.h>

#include "ldap.h"

#include "ntl.h"
#include "ntlc.h"
#include "ntg.h"
#include "ntg_keys.h"
#include "progname.h"

extern int ntlc_error;
extern int ntg_keys_error;

static const char rcsid[] = "$Id: 438024f3814d6719bb6c50806da03226a6b7be56 $";

static void usage(void);

static void
usage(void)
{

	(void)printf("Usage: %s [-h] [ -f cfg_file ] [ -p profile ] [basedn]\n\n", getprogname());

	(void)printf("Copyright (c) 2006, 2007, 2008 Niklas Olmes <niklas@noxa.de>\n"
		     "                               http://www.nevm.de/ntldap/\n"
		     "                               NEVM-RIPE\n"
		     "Licensed under the GNU GPL Version 2.\n");
}

int
main(int argc, char *argv[])
{
	NTG_CTX *ctx;
	NTL_CFG *cfg;
	char profile[256], cfgfile[256];
	char *basedn;
	int c;

	setprogname(argv[0]);

	*profile = *cfgfile = '\0';
	basedn = NULL;

	if (argc > 1)
		while ((c = getopt(argc, argv, "f:p:h")) != -1) {
			switch(c) {
			case 'f':
				(void)snprintf(cfgfile, sizeof(cfgfile), "%s", optarg);
				break;
			case 'p':
				(void)snprintf(profile, sizeof(profile), "%s", optarg);
				break;
			case 'h':
			default:
				usage();
				exit(127);
			}
		}
	argc -= optind;
	argv += optind;

	if (argc > 1) {
		(void)fprintf(stderr, "Error: Too many arguments.\n");
		exit(127);
	} else if (argc == 1)
		basedn = argv[0];

	if ((cfg = ntlc_cfg_read(*cfgfile == '\0' ? NULL : cfgfile, *profile == '\0' ? NULL : profile)) == NULL) {
		ntlc_perror(argv[0]);
		return ntlc_error;
	}

	if ((ctx = ntg_ctx_init()) == NULL)
		errx(125, "error initializing ctx");

	ctx->keys = ntg_keys_init_default();
	if (ntg_keys_cfg_read(ctx->keys, *cfgfile == '\0' ? NULL : cfgfile) != 0) {
		ntg_keys_perror(argv[0]);
		return ntg_keys_error;
	}

	if (ntg_init_curses(ctx) != 0) {
		ntg_ctx_destroy(ctx);
		warnx("\r\nCannot init.  Press key.\r\n");
		(void)getchar();
		errx(126, "unable to initialize");
	}

	ctx->cfg = cfg;
	if (ntg_init_ldap(ctx) != 0) {
		ntg_ctx_destroy(ctx);
		ntg_cleanup();
		errx(127, "unable to init LDAP connection");
	}

	(void)ntg_init_schema(ctx); /* ignore errors here */

	(void)ntg_search(ctx, basedn == NULL ? ctx->cfg->basedn : basedn, LDAP_SCOPE_ONELEVEL, "objectClass=*", ntg_pager_generic);

	ntg_ctx_destroy(ctx);
	ntg_cleanup();
	return EXIT_SUCCESS;
}
